#include "gbasys.h"
#include <math.h>

#define OAM			((Sprite*)0x07000000)
#define OBJCHARDATA	((u8*)0x06010000)
#define BGCHARDATA	((u8*)0x06004000)
#define OBJ_PLTT	((u16*)0x05000200)
#define BG_PLTT		((u16*)0x05000000)

//typedef s32 fixed;
#define fixed u32
#define INT_TO_FIXED(x)         ((x) << 16)
#define FIXED_TO_INT(x)         (u16)((x) >> 16)
#define DOUBLE_TO_FIXED(x)      ((long)(x * 65536.0 + 0.5))
#define FIXED_TO_DOUBLE(x)      (((double)x) / 65536.0)
#define ROUND_FIXED_TO_INT(x)   (((x) + 0x8000) >> 16)

#define PI 3.14159
#define RADIAN(n) (((float)n)/(float)180*PI)
#define DEGREE(n) ((float)n*PI/(float)180)

typedef struct _Sprite { u16 Attrib0, Attrib1, Attrib2, RotateScale; } Sprite;		//define a sprite as a structure

extern u8 pic;									//adds the pic and palette from data.asm
extern u16 pal;

//u16 *m0=(u16*)0x6000000;						//tells the compiler when our bkg map is going to be located

u16 bghofs = 0;
u16 bgvofs = 0;
u8 plyX = 1;
u8 plyY = 1;

//fixed COS[8] = {0x00010000, 0x0000B504, 0x00000000, 0xFFFF4AFB, 0xFFFFFFFF, 0xFFFF4AFB, 0x00000000, 0x0000B504 };
//fixed SIN[8] = {0x00000000, 0x0000B504, 0x00010000, 0x0000B504, 0x00000000, 0xFFFF4AFB, 0xFFFFFFFF, 0xFFFF4AFB };

//u16 COS[8] = {0x0100, 0x00B5, 0x0000, 0xFF4A, 0xFFFF, 0xFF4A, 0x0000, 0x00B5 };
//u16 SIN[8] = {0x0000, 0x00B5, 0x0100, 0x00B5, 0x0000, 0xFF4A, 0xFFFF, 0xFF4A };

s16 COS[256];
s16 SIN[256];

void DMA3Call(u32 Src,u32 Dst,u32 Cnt)	//used to load our sprite info into vram			
{
	REG_DM3SAD = Src;						//where the data is
	REG_DM3DAD = Dst;						//where the data goes
	REG_DM3CNT = Cnt;						//how much data should be loaded
}

// VSYNC function by Eloist
void WaitForVSync()
{
  __asm
   {
    mov 	r0, #0x4000006
    scanline_wait:
     ldrh	r1, [r0]
     cmp	r1, #160
    bne 	scanline_wait
   }
}

void PlaceSprite(s16 x,s16 y,u16 shape, u16 size, u16 tile, u16 i, u16 scale, u8 angle)
{										//tile # to use
	OAM[i].Attrib0 = 0x2000+shape+y;	//loads sprite 0 at y coordinate.  the 0x2000 tells it to use a 256 color palette
	OAM[i].Attrib1 = size+x;			//loads sprite 0 at x coordinate
	OAM[i].Attrib2 = tile;				//sets up sprite tile #

	OAM[0].RotateScale = ((scale*COS[angle])>>8);
	OAM[1].RotateScale = ((scale*SIN[angle])>>8);
	OAM[2].RotateScale = 65535-((scale*SIN[angle])>>8);
	OAM[3].RotateScale = ((scale*COS[angle])>>8);
}

void Init()
{
	u16 i;

	REG_DISPCNT = 0x1040;

	DMA3Call((u32)&pal, (u32)OBJ_PLTT, 0x84000080);				// Setup the sprites palette
//	DMA3Call((u32)&pal, (u32)BG_PLTT, 0x84000080);				// Setup the background palette
	DMA3Call((u32)&pic, (u32)OBJCHARDATA, 0x84000000+64*4);		// Copy Sprites Character Bloc to VRAM
//	DMA3Call((u32)&pic, (u32)BGCHARDATA, 0x84000000+64*52);		// Copy Background Character Bloc into VRAM

	for(i=0;i<256;i++) {
		COS[i] = (s16)(cos(i*PI/128)*256);
		SIN[i] = (s16)(sin(i*PI/128)*256);
	}

	PlaceSprite(32,32,0x0300,0x8000,0,0,0x0100,0);
}

void C_Entry()
{
	u8 angle=0;
	u16 scale=0x0100;

	Init();										//call init to set up our screen display
	while (1)
	{
		if(!(REG_P1 & BIT04)) angle+=1;
		if(!(REG_P1 & BIT05)) angle-=1;
		if(!(REG_P1 & BIT06)) scale+=1;
		if(!(REG_P1 & BIT07)) scale-=1;
		WaitForVSync();
		PlaceSprite(32,32,0x0300,0x8000,0,0,scale,angle);
	}
}

//void GetKeys()
//{
  // Note that I reset the bit after checking weather the key was pressed. See the 
  // Programming Matters section in the readme.
//  if( !(P1 & BIT00) ) { Greetings = 1;  P1 |= BIT00; }   // "A" button
//  if( !(P1 & BIT01) ) { Greetings = 2;  P1 |= BIT01; }   // "B" button
//  if( !(P1 & BIT02) ) { Greetings = 3;  P1 |= BIT02; }   // "Select" button
//  if( !(P1 & BIT03) ) { Greetings = 4;  P1 |= BIT03; }   // "Start" button 
//  if( !(P1 & BIT04) ) { Greetings = 8;  P1 |= BIT04; }   // Right-Direction
//  if( !(P1 & BIT05) ) { Greetings = 7;  P1 |= BIT05; }   // Left-Direction
//  if( !(P1 & BIT06) ) { Greetings = 5;  P1 |= BIT06; }   // Up-Direction
//  if( !(P1 & BIT07) ) { Greetings = 6;  P1 |= BIT07; }   // Down-Direction
//  if( !(P1 & BIT08) ) { Greetings = 10; P1 |= BIT08; }  // "R" button
//  if( !(P1 & BIT09) ) { Greetings = 9;  P1 |= BIT09; }  // "L" button
//}